Package: FossilSimShiny
Title: Shiny Application for 'FossilSim'
Version: 1.1.2
Authors@R: c(person("Titouan", "Chambe", email="titouan.chambe@u-psud.fr", role=c("aut")),
    person("Joelle", "Barido-Sottani", email="joelle.barido-sottani@m4x.org",role=c("aut","cre","cph")),
    person("Rachel", "Warnock", email="rachel.warnock@fau.de", role=c("aut","cph"))
    )
Description: A shiny application based on 'FossilSim'. Used for simulating
    tree, taxonomic and fossil data under mechanistic models of speciation,
    preservation and sampling.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.1.2
Imports: shiny (>= 1.7.1), shinyjs, FossilSim, TreeSim
Suggests: knitr, rmarkdown, spelling
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2024-04-29 11:24:28 UTC; barid
Author: Titouan Chambe [aut],
  Joelle Barido-Sottani [aut, cre, cph],
  Rachel Warnock [aut, cph]
Maintainer: Joelle Barido-Sottani <joelle.barido-sottani@m4x.org>
Repository: CRAN
Date/Publication: 2024-04-29 11:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:42:05 UTC; unix
