## FossilSimShiny

`R` package for exploring phylogenetic trees, taxonomy and fossil data simulations using a simple shiny interface.

The latest version can be installed in `R` using the package `devtools`:

```{R, eval = FALSE}
  library(devtools)
  install_github("fossilsim/shiny")
```

### Quick start

Launch the application using `FossilSimShiny::launchFossilSimShiny()`.
